@@ -402,6 +402,7 @@ static int send_request_body(ne_request *req, int retry)
     }
 
     if (bytes == 0) {
+        NE_DEBUG(NE_DBG_CORE, "Request body sent successfully");
         return NE_OK;
     } else {
         NE_DEBUG(NE_DBG_HTTP, "Request body provider failed with "
@@ -426,21 +427,17 @@ static void add_fixed_headers(ne_request *req)
      * servers to try harder to get a persistent connection, except if
      * using a proxy as per 2068§19.7.1.  Always add TE: trailers. */
     if (!sess->flags[NE_SESSFLAG_PERSIST]) {
-       ne_buffer_czappend(req->headers, "Connection: TE, close" EOL);
-    } 
+       ne_buffer_czappend(req->headers, "Connection: close" EOL);
+    }
     else if (!sess->is_http11 && !sess->any_proxy_http) {
-        ne_buffer_czappend(req->headers, 
+        ne_buffer_czappend(req->headers,
                            "Keep-Alive: " EOL
-                          "Connection: TE, Keep-Alive" EOL);
-    } 
+                          "Connection: Keep-Alive" EOL);
+    }
     else if (!req->session->is_http11 && !sess->any_proxy_http) {
-        ne_buffer_czappend(req->headers, 
+        ne_buffer_czappend(req->headers,
                            "Keep-Alive: " EOL
-                           "Proxy-Connection: Keep-Alive" EOL
-                           "Connection: TE" EOL);
-    } 
-    else {
-        ne_buffer_czappend(req->headers, "Connection: TE" EOL);
+                           "Proxy-Connection: Keep-Alive" EOL);
     }
 
     ne_buffer_concat(req->headers, "TE: trailers" EOL "Host: ", 
