@@ -35,6 +35,11 @@
 #include <sys/socket.h>
 #endif
 
+#ifdef __linux__
+#include <sys/ioctl.h>
+#include <linux/sockios.h>
+#endif
+
 #ifdef NE_USE_POLL
 #include <sys/poll.h>
 #elif defined(HAVE_SYS_SELECT_H)
@@ -173,6 +178,8 @@ typedef struct in_addr ne_inet_addr;
 
 /* Socket read timeout */
 #define SOCKET_READ_TIMEOUT 120
+/* Socket step timeout */
+#define SOCKET_STEP_TIMEOUT 10
 
 /* Critical I/O functions on a socket: useful abstraction for easily
  * handling SSL I/O alongside raw socket I/O. */
@@ -506,10 +513,50 @@ static int readable_raw(ne_socket *sock, int secs)
     return (ret == 0) ? NE_SOCK_TIMEOUT : 0;
 }
 
+#ifdef __linux__
+static int wait_pending_writes(ne_socket *sock, int stepwait)
+{
+    /* If there's any data in the TCP queue,
+       wait until it's been drained */
+
+    int prevpending = INT_MAX, pending = INT_MAX;
+    ssize_t ret;
+    do {
+        /* Ready to read from socket? */
+        ret = readable_raw(sock, stepwait);
+        if(ret == NE_SOCK_ERROR) return ret;
+
+        /* There was a timeout.. */
+        if(ret == NE_SOCK_TIMEOUT) {
+            /* Is there any data pending for writing? */
+            int ioret = ioctl(sock->fd, SIOCOUTQ, &pending);
+            if(ioret == -1) {
+                set_strerror(sock, ne_errno);
+                return NE_SOCK_ERROR;
+            }
+            if(pending > 0 && pending < prevpending) {
+                if(prevpending == INT_MAX) prevpending = -1;
+                NE_DEBUG(NE_DBG_SOCKET, "Pending bytes on TCP send buffer, waiting. (%d bytes)", pending);
+                prevpending = pending;
+            }
+            else {
+                /* Timeout, no change in buffer size since last time.
+                   Connection is probably hung */
+                return NE_SOCK_TIMEOUT;
+            }
+        }
+    } while(ret == NE_SOCK_TIMEOUT);
+    return 0;
+}
+#endif
+
 static ssize_t read_raw(ne_socket *sock, char *buffer, size_t len)
 {
     ssize_t ret;
-    
+#ifdef __linux__
+    ret = wait_pending_writes(sock, SOCKET_STEP_TIMEOUT);
+    if (ret) return ret;
+#endif
     ret = readable_raw(sock, sock->rdtimeout);
     if (ret) return ret;
 
@@ -1795,65 +1842,50 @@ int ne_sock_connect_ssl(ne_socket *sock, ne_ssl_context *ctx, void *userdata)
     }
 */
 
-    // non-blocking SSL handshake 
-    int ready = 0;
-    int expired = 0;
-    int errsv = 0;
+    // non-blocking SSL handshake
     struct timeval tv;
 
     while((ret = SSL_connect(ssl)) != 1){
-       
         fd_set fds;
         FD_ZERO(&fds);
         FD_SET(sock->fd, &fds);
 
-        do{
-            tv.tv_sec = 1;
-            tv.tv_usec = 0;
-            switch(SSL_get_error(ssl, ret)){
-                case SSL_ERROR_WANT_WRITE:
-                    ready = select(sock->fd +1, NULL, &fds, NULL, &tv);
-                    break;
+        tv.tv_sec = 1;
+        tv.tv_usec = 0;
 
-                case SSL_ERROR_WANT_READ:
-                    ready = select(sock->fd +1, &fds, NULL, NULL, &tv);
-                    break;
-
-                case SSL_ERROR_WANT_X509_LOOKUP:
-                    ready = select(sock->fd +1, &fds, &fds, NULL, &tv);
-                    break;
-
-                case SSL_ERROR_NONE:
-                case SSL_ERROR_ZERO_RETURN:
-                case SSL_ERROR_WANT_CONNECT:
-                case SSL_ERROR_WANT_ACCEPT:
-                case SSL_ERROR_SYSCALL:
-                    break;
-                
-                case SSL_ERROR_SSL:{
-                    int ssl_err_code = ERR_get_error();
-                    char* ssl_err_message = ERR_reason_error_string(ssl_err_code);
-                    ERR_clear_error();
-                    set_error(sock, _(ssl_err_message));
-                    return NE_SOCK_ERROR;
-                }
-
-                default: 
-                    set_error(sock, _("Unknown error during SSL handshake"));
-                    return NE_SOCK_ERROR;
-            }
-
-            if(ready == -1)
-                errsv = errno;
-
-            if(time(0) > timeout){
-                expired = 1;
+        switch(SSL_get_error(ssl, ret)){
+            case SSL_ERROR_WANT_WRITE:
+                select(sock->fd +1, NULL, &fds, NULL, &tv);
                 break;
+
+            case SSL_ERROR_WANT_READ:
+                select(sock->fd +1, &fds, NULL, NULL, &tv);
+                break;
+
+            case SSL_ERROR_WANT_X509_LOOKUP:
+                select(sock->fd +1, &fds, &fds, NULL, &tv);
+                break;
+
+            case SSL_ERROR_NONE:
+            case SSL_ERROR_ZERO_RETURN:
+            case SSL_ERROR_WANT_CONNECT:
+            case SSL_ERROR_WANT_ACCEPT:
+            case SSL_ERROR_SYSCALL:
+                break;
+            case SSL_ERROR_SSL:{
+                int ssl_err_code = ERR_get_error();
+                const char* ssl_err_message = ERR_reason_error_string(ssl_err_code);
+                ERR_clear_error();
+                set_error(sock, _(ssl_err_message));
+                return NE_SOCK_ERROR;
             }
 
-        }while((ready == 0) || (errsv == EINTR) || (errsv == EWOULDBLOCK)); // loop until there is something in the socket or timed out
+            default:
+                set_error(sock, _("Unknown error during SSL handshake"));
+                return NE_SOCK_ERROR;
+        }
 
-        if(expired){
+        if(time(0) > timeout){
             error_ossl(sock, ret);
             SSL_free(ssl);
             sock->ssl = NULL;
