@@ -132,6 +132,54 @@ public:
     ///
     const std::pair<AwsSecretKey, AwsAccessKey> & getAwsAutorizationKeys() const;
 
+    ///
+    /// \brief define a Amazon S3 bucket region
+    /// \param region the region
+    ///
+    void setAwsRegion(const AwsRegion & region);
+
+    ///
+    /// \brief get Amazon S3 bucket region
+    /// \return the bucket region
+    ///
+    const AwsRegion & getAwsRegion() const;
+
+    ///
+    /// \brief define an Amazon S3 security token
+    /// \param token the security token
+    ///
+    void setAwsToken(const AwsToken & token);
+
+    ///
+    /// \brief get Amazon S3 security token
+    /// \return the security token
+    ///
+    const AwsToken & getAwsToken() const;
+
+    ///
+    /// \brief set whether we're using an S3 path-based url
+    /// \param alternate whether using an S3 path-based url
+    ///
+    void setAwsAlternate(const bool & alternate);
+
+    ///
+    /// \brief get whether we're using an S3 path-based url
+    /// \return whether we're using an S3 path-based url
+    ///
+    const bool & getAwsAlternate() const;
+
+    ///
+    /// \brief set the secret key for Azure authentication
+    /// \param key the secret key
+    ///
+    void setAzureKey(const AzureSecretKey & key);
+
+    ///
+    /// \brief get the secret key used for Azure authentication
+    /// \return the secret key
+    ///
+    const AzureSecretKey & getAzureKey() const;
+
     /// set listing mode flag for S3 bucket
     void setS3ListingMode(const S3ListingMode::S3ListingMode s3_listing_mode);
 
