@@ -26,10 +26,12 @@
 #include <xml/davdeletexmlparser.hpp>
 #include <xml/metalinkparser.hpp>
 #include <xml/s3propparser.hpp>
+#include <xml/azurepropparser.hpp>
 
 #include <utils/davix_logger_internal.hpp>
 #include <utils/davix_utils_internal.hpp>
 #include <utils/davix_s3_utils.hpp>
+#include <utils/davix_azure_utils.hpp>
 
 #include <request/httprequest.hpp>
 #include <fileops/fileutils.hpp>
@@ -47,6 +49,7 @@ static const std::string simple_listing("<propfind xmlns=\"DAV:\"><prop><resourc
 static const std::string stat_listing("<?xml version=\"1.0\" encoding=\"utf-8\" ?><D:propfind xmlns:D=\"DAV:\" xmlns:L=\"LCGDM:\"><D:prop>"
                                       "<D:displayname/><D:getlastmodified/><D:creationdate/><D:getcontentlength/>"
                                       "<D:resourcetype><D:collection/></D:resourcetype><L:mode/>"
+                                      "<D:owner></D:owner><D:group></D:group>"
                                       "</D:prop>"
                                       "</D:propfind>");
 
@@ -67,22 +70,16 @@ struct DirHandle{
   return a vector with the content of the request if success
 */
 std::vector<char> req_webdav_propfind(HttpRequest* req, DavixError** err){
-    DavixError* tmp_err=NULL;
     int ret =-1;
     std::vector<char> res;
 
     req->addHeaderField("Depth","0");
     req->setRequestMethod("PROPFIND");
 
-    if( (ret = req->executeRequest(&tmp_err)) ==0){
-        ret = davixRequestToFileStatus(req, davix_scope_stat_str(), &tmp_err);
+    if( (ret = req->executeRequest(err)) ==0){
         res.swap(req->getAnswerContentVec());
-
     }
 
-    if(ret != 0)
-        DavixError::propagateError(err, tmp_err);
-
     return res;
 }
 
@@ -147,7 +144,6 @@ int dav_stat_mapper_http(Context& context, const RequestParams* params, const Ur
 
 
 dav_ssize_t incremental_listdir_parsing(HttpRequest* req, XMLPropParser * parser, dav_size_t s_buff, const std::string & scope){
-  //  std::cout << "time 1 pre-fecth" << time(NULL) << std::endl;
     DavixError* tmp_err=NULL;
 
     char buffer[s_buff+1];
@@ -273,7 +269,10 @@ int internal_move(Context & c, const Uri & url, const RequestParams & params, co
     if(tmp_err == NULL){
         req.setParameters(params);
         req.setRequestMethod("MOVE");
-        req.addHeaderField("Destination", target_url);
+
+        Uri uri(target_url);
+        uri.httpizeProtocol();
+        req.addHeaderField("Destination", uri.getString());
 
         if( (ret = req.executeRequest(&tmp_err)) == 0){
                 parse_creation_deletion_result(req.getRequestCode(),  url, davix_scope_mv_str(), req.getAnswerContentVec());
@@ -301,7 +300,7 @@ int internal_checksum(Context & c, const Uri & url, const RequestParams *p, std:
         req.addHeaderField("Want-Digest", chk_algo);
         req.setParameters(params);
         if( (ret = req.executeRequest(&tmp_err)) == 0
-            && (ret = davixRequestToFileStatus(&req, davix_scope_mkdir_str(), &tmp_err)) >=0){
+            && !tmp_err && (ret = davixRequestToFileStatus(&req, davix_scope_mkdir_str(), &tmp_err)) >=0){
 
             // try simple MD5 ( standard )
             if(compare_ncase(chk_algo, "MD5") == 0){
@@ -490,9 +489,18 @@ static bool is_s3_operation(IOChainContext & context){
     return ( proto.compare(0, 2, "s3") ==0 || protocol_flag == RequestProtocol::AwsS3);
 }
 
-void internal_s3_creat_bucket(Context & c, const Uri & url, const RequestParams & params){
+static void internal_s3_create_bucket_or_dir(Context & c, const Uri & url, const RequestParams & params){
     DavixError * tmp_err=NULL;
-    PutRequest req(c, url, &tmp_err);
+
+    /* make sure path ends with a slash, otherwise s3
+       will just create a zero-length file */
+    Uri url2 = url;
+    if(url.getPath()[url.getPath().size()-1] != '/') {
+        url2.setPath(url.getPath() + "/");
+    }
+
+    PutRequest req(c, url2, &tmp_err);
+    req.addHeaderField("Content-Length", "0");
     checkDavixError(&tmp_err);
 
     req.setParameters(params);
@@ -504,15 +512,69 @@ void internal_s3_creat_bucket(Context & c, const Uri & url, const RequestParams
     checkDavixError(&tmp_err);
 }
 
+void S3MetaOps::move(IOChainContext & iocontext, const std::string & target_url) {
+    const std::string scope = "Davix::S3MetaOps::move";
+    if(!is_s3_operation(iocontext)) {
+        return HttpIOChain::move(iocontext, target_url);
+    }
+
+    Context context = iocontext._context;
+    RequestParams params = iocontext._reqparams;
+    Uri uri(iocontext._uri);
+    Uri target(target_url);
+
+    // verify both are using the same s3 provider/server
+    std::string p1 = S3::extract_s3_provider(uri);
+    std::string p2 = S3::extract_s3_provider(target);
+
+    if(p1 != p2) {
+        throw DavixException(scope, StatusCode::OperationNonSupported,
+                             "It looks that the two URLs are not using the same S3 provider. Unable to perform the move operation.");
+    }
+
+    std::string source_bucket = S3::extract_s3_bucket(uri, params.getAwsAlternate());
+    std::string source_path = S3::extract_s3_path(uri, params.getAwsAlternate());
+
+    DavixError *tmp_err = NULL;
+    PutRequest req(context, target, &tmp_err);
+    checkDavixError(&tmp_err);
+    req.setParameters(iocontext._reqparams);
+    req.addHeaderField("x-amz-copy-source", "/" + source_bucket + source_path);
+
+    req.executeRequest(&tmp_err);
+    checkDavixError(&tmp_err);
+
+    // if copying was successful, delete the source file
+    if(req.getRequestCode() == 200) {
+        std::string region = S3::detect_region(uri);
+        DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, "Detected region for source endpoint: " + region);
+        checkDavixError(&tmp_err);
+
+        DeleteRequest req(context, uri, &tmp_err);
+        checkDavixError(&tmp_err);
+
+        RequestParams p(iocontext._reqparams);
+        p.setAwsRegion(region);
+        req.setParameters(p);
+
+        req.executeRequest(&tmp_err);
+        checkDavixError(&tmp_err);
+    }
+    else {
+        std::stringstream str;
+        str << "Received code " << req.getRequestCode() << " when trying to copy file - will not perform deletion";
+        throw DavixException(scope, StatusCode::UnknowError, str.str());
+    }
+}
+
 
 void S3MetaOps::checksum(IOChainContext &iocontext, std::string &checksm, const std::string &chk_algo){
     internal_checksum(iocontext._context, iocontext._uri, iocontext._reqparams, checksm, chk_algo);
 }
 
 void S3MetaOps::makeCollection(IOChainContext &iocontext){
-
     if(is_s3_operation(iocontext)){
-        internal_s3_creat_bucket( iocontext._context, iocontext._uri, iocontext._reqparams);
+        internal_s3_create_bucket_or_dir( iocontext._context, iocontext._uri, iocontext._reqparams);
     }else{
         HttpIOChain::makeCollection(iocontext);
     }
@@ -536,9 +598,10 @@ void s3StatMapper(Context& context, const RequestParams* params, const Uri & uri
 
         // if 404, target either doesn't exist or is a S3 "directory"
         if(code == 404){
+            DavixError::clearError(&tmp_err);
             // try to "list" target resource and see if there is anything inside it, if there is, then it's a directory
             Uri new_url = S3::s3UriTransformer(uri, p, true);
-            DirHandle handle(new GetRequest(context, new_url, &tmp_err), new S3PropParser(params->getS3ListingMode(), uri.getPath()));
+            DirHandle handle(new GetRequest(context, new_url, &tmp_err), new S3PropParser(params->getS3ListingMode(), S3::extract_s3_path(uri, params->getAwsAlternate())));
 
             dav_ssize_t s_resu=0;
 
@@ -581,12 +644,14 @@ void s3StatMapper(Context& context, const RequestParams* params, const Uri & uri
             st_info.mode |= S_IFDIR;
         }
         else if(code == 200){ // found something, must be a file not directory
-            std::string length;
-            std::string type;
             st_info.mode = 0755;
 
-            if(uri.getPath() == "/") // is bucket
+            std::string s3_path = S3::extract_s3_path(uri, params->getAwsAlternate());
+            if(s3_path == "/") // is bucket
                 st_info.mode |= S_IFDIR;
+            else if(s3_path[s3_path.size()-1] == '/' && req.getAnswerSize() == 0) { // is a directory
+                st_info.mode |= S_IFDIR;
+            }
             else{   // is file
                 st_info.mode |= S_IFREG;
                 const dav_ssize_t s = req.getAnswerSize();
@@ -653,12 +718,12 @@ void s3_start_listing_query(Ptr::Scoped<DirHandle> & handle, Context & context,
     DavixError* tmp_err=NULL;
 
     if(params->getS3ListingMode() == S3ListingMode::Hierarchical){
-        Uri new_url = S3::s3UriTransformer(url, params, true); 
-        handle.reset(new DirHandle(new GetRequest(context, new_url, &tmp_err), new S3PropParser(params->getS3ListingMode(), url.getPath())));
+        Uri new_url = S3::s3UriTransformer(url, params, true);
+        handle.reset(new DirHandle(new GetRequest(context, new_url, &tmp_err), new S3PropParser(params->getS3ListingMode(), S3::extract_s3_path(url, params->getAwsAlternate()))));
     }
     else if(params->getS3ListingMode() == S3ListingMode::SemiHierarchical){
-        Uri new_url = S3::s3UriTransformer(url, params, false); 
-        handle.reset(new DirHandle(new GetRequest(context, new_url, &tmp_err), new S3PropParser(params->getS3ListingMode(), url.getPath())));
+        Uri new_url = S3::s3UriTransformer(url, params, false);
+        handle.reset(new DirHandle(new GetRequest(context, new_url, &tmp_err), new S3PropParser(params->getS3ListingMode(), S3::extract_s3_path(url, params->getAwsAlternate()))));
     }
     else{
         if(is_a_bucket(url) == false){
@@ -669,6 +734,7 @@ void s3_start_listing_query(Ptr::Scoped<DirHandle> & handle, Context & context,
     checkDavixError(&tmp_err);
 
 
+
     const int operation_timeout = params->getOperationTimeout()->tv_sec;
     HttpRequest & http_req = *(handle->request);
     XMLPropParser & parser = *(handle->parser);
@@ -683,7 +749,7 @@ void s3_start_listing_query(Ptr::Scoped<DirHandle> & handle, Context & context,
     check_file_status(http_req, davix_scope_directory_listing_str());
 
     size_t prop_size = 0;
-    do{ // first entry -> bucket informations
+    do{ // first entry -> bucket information
        s_resu = incremental_listdir_parsing(&http_req, &parser, 2048, davix_scope_directory_listing_str());
 
        prop_size = parser.getProperties().size();
@@ -727,5 +793,171 @@ bool S3MetaOps::nextSubItem(IOChainContext &iocontext, std::string &entry_name,
 
 }
 
+/////////////////////////
+/////////////////////////
+
+AzureMetaOps::AzureMetaOps() : HttpIOChain()
+ {}
+
+AzureMetaOps::~AzureMetaOps(){}
+
+static bool is_azure_operation(IOChainContext & context){
+    return context._reqparams->getProtocol() == RequestProtocol::Azure;
+}
+
+void azureStatMapper(Context& context, const RequestParams* params, const Uri & uri, struct StatInfo& st_info) {
+    const std::string scope = "Davix::azureStatMapper";
+    DavixError * tmp_err=NULL;
+
+    //Context c;
+    HeadRequest req(context, uri, &tmp_err);
+
+    // we need to modify it, hence copy
+    RequestParams p(params);
+    // we just need to know if target has anything inside it
+    //p.setS3MaxKey(1);
+
+    if( tmp_err == NULL){
+        req.setParameters(p);
+        req.executeRequest(&tmp_err);
+        const int code = req.getRequestCode();
+
+        // if 404, target either doesn't exist or is an Azure "directory". TODO: add support to stat directories. must have func tests by then
+        if(code == 404){
+            DavixError::clearError(&tmp_err);
+            Uri new_url = Azure::transformURI(uri, p, true);
+            DirHandle handle(new GetRequest(context, new_url, &tmp_err), new AzurePropParser(Azure::extract_azure_filename(uri)));
+
+            dav_ssize_t s_resu=0;
+
+            const int operation_timeout = p.getOperationTimeout()->tv_sec;
+            HttpRequest & http_req = *(handle.request);
+            XMLPropParser & parser = *(handle.parser);
+
+            time_t timestamp_timeout = time(NULL) + ((operation_timeout)?(operation_timeout):180);
+
+            http_req.setParameters(p);
+
+            http_req.beginRequest(&tmp_err);
+            checkDavixError(&tmp_err);
+            check_file_status(http_req, scope);
+
+            size_t prop_size = 0;
+            do{ // first entry -> container information
+                s_resu = incremental_listdir_parsing(&http_req, &parser, 2048, davix_scope_directory_listing_str());
+
+                prop_size = parser.getProperties().size();
+                if(s_resu < 2048 && prop_size <1){ // verify request status : if req done + no data -> error
+                    throw DavixException(davix_scope_directory_listing_str(), StatusCode::IsNotADirectory, "The specified directory does not exist");
+                }
+                if(timestamp_timeout < time(NULL)){
+                    throw DavixException(davix_scope_directory_listing_str(), StatusCode::OperationTimeout, "Operation timeout triggered while directory listing");
+                }
+            }while( prop_size < 1); // prop < 1 means not enough data
+
+            st_info.mode = 0755;
+            st_info.mode |= S_IFDIR;
+        }
+        // file exists, parse its info
+        else if(code == 200) {
+            st_info.mode = 0755;
+            st_info.mode |= S_IFREG;
+            const dav_ssize_t s = req.getAnswerSize();
+            st_info.size = std::max<dav_ssize_t>(0,s);
+            st_info.mtime = req.getLastModified();
+        }
+
+    }
+
+}
+
+StatInfo & AzureMetaOps::statInfo(IOChainContext & iocontext, StatInfo & st_info) {
+    if(is_azure_operation(iocontext)) {
+        azureStatMapper(iocontext._context, iocontext._reqparams, iocontext._uri, st_info);
+        return st_info;
+    }
+    else {
+        return HttpIOChain::statInfo(iocontext, st_info);
+    }
+}
+
+static void azure_start_listing_query(Ptr::Scoped<DirHandle> & handle, Context & context, const RequestParams* params, const Uri & url, const std::string & body) {
+    DavixError* tmp_err=NULL;
+    dav_ssize_t s_resu;
+
+    Uri new_url = Davix::Azure::transformURI(url, params, true);
+    handle.reset(new DirHandle(new GetRequest(context, new_url, &tmp_err), new AzurePropParser(Davix::Azure::extract_azure_filename(url))));
+
+    const int operation_timeout = params->getOperationTimeout()->tv_sec;
+    HttpRequest & http_req = *(handle->request);
+    XMLPropParser & parser = *(handle->parser);
+    time_t timestamp_timeout = time(NULL) + ((operation_timeout)?(operation_timeout):180);
+
+    http_req.setParameters(params);
+
+    http_req.beginRequest(&tmp_err);
+    checkDavixError(&tmp_err);
+
+    check_file_status(http_req, davix_scope_directory_listing_str());
+
+    size_t prop_size = 0;
+    do{ // first entry -> container information
+       s_resu = incremental_listdir_parsing(&http_req, &parser, 2048, davix_scope_directory_listing_str());
+
+       prop_size = parser.getProperties().size();
+       if(s_resu < 2048 && prop_size <1){ // verify request status : if req done + no data -> error
+           throw DavixException(davix_scope_directory_listing_str(), StatusCode::IsNotADirectory, "The specified directory does not exist");
+       }
+       if(timestamp_timeout < time(NULL)){
+          throw DavixException(davix_scope_directory_listing_str(), StatusCode::OperationTimeout, "Operation timeout triggered while directory listing");
+       }
+    }while( prop_size < 1); // prop < 1 means not enough data
+
+}
+
+bool azure_get_next_property(Ptr::Scoped<DirHandle> & handle, std::string & name_entry, StatInfo & info) {
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, " -> azure_get_next_property");
+    const size_t read_size = 2048;
+
+    HttpRequest& req = *(handle->request); // setup env again
+    XMLPropParser& parser = *(handle->parser);
+
+    size_t prop_size = parser.getProperties().size();
+    ssize_t s_resu = read_size;
+
+    while( prop_size == 0
+          && s_resu > 0){ // execute request only if no property are available
+
+        // continue the parsing until one more result
+       s_resu = incremental_listdir_parsing(&req, &parser, read_size, "S3::listing");
+       prop_size = parser.getProperties().size();
+    }
+
+    if(prop_size == 0){
+        return false; // end of the request, end of the story
+    }
+
+    FileProperties & front = parser.getProperties().front();
+    name_entry.swap(front.filename);
+    info = front.info;
+    parser.getProperties().pop_front(); // clean the current element
+    return true;
+}
+
+static bool azure_directory_listing(Ptr::Scoped<DirHandle> & handle, Context & context, const RequestParams* params, const Uri & uri, const std::string & body, std::string & name_entry, StatInfo & info){
+    if(handle.get() == NULL){
+        azure_start_listing_query(handle, context, params, uri, body);
+    }
+    return azure_get_next_property(handle, name_entry, info);
+}
+
+bool AzureMetaOps::nextSubItem(IOChainContext &iocontext, std::string &entry_name, StatInfo &info) {
+    if(is_azure_operation(iocontext)){
+        return azure_directory_listing(directoryItem, iocontext._context, iocontext._reqparams, iocontext._uri, stat_listing,
+                                 entry_name, info);
+    }else{
+        return HttpIOChain::nextSubItem(iocontext, entry_name, info);
+    }
+}
 
 } // Davix
