@@ -147,6 +147,7 @@ NEONRequest::NEONRequest(HttpRequest & h, Context& context, const Uri & uri_req)
     _current( new Uri(uri_req)),
     _orig(_current),
     _number_try(0),
+    _redirects(0),
     _total_read_size(0),
     _last_read(-1),
     _vec(),
@@ -199,6 +200,12 @@ int NEONRequest::createRequest(DavixError** err){
     }
 
     boost::shared_ptr<Uri> redir_url = ContextExplorer::SessionFactoryFromContext(_c).redirectionResolve(_request_type, *_current);
+
+    // performing an operation which could change the PFN? Clear all cache entries for selected URL
+    if(_request_type == "DELETE" || _request_type == "MOVE") {
+        ContextExplorer::SessionFactoryFromContext(_c).redirectionClean(*_current.get());
+    }
+
     if(redir_url.get() != NULL){
         _current.swap(redir_url);
     }
@@ -206,6 +213,18 @@ int NEONRequest::createRequest(DavixError** err){
     if( instanceSession(err) < 0)
         return -1;
 
+
+    // reconfigure protos
+    configureRequestParamsProto(*_current, params);
+
+    // configure S3 params if needed
+    if(params.getProtocol() == RequestProtocol::AwsS3)
+        configureS3params();
+
+    // configure azure params if needed
+    if(params.getProtocol() == RequestProtocol::Azure)
+        configureAzureParams();
+
     _req= ne_request_create(_neon_sess->get_ne_sess(), _request_type.c_str(), _current->getPathAndQuery().c_str());
     configureRequest();
 
@@ -239,14 +258,21 @@ bool NEONRequest::checkTimeout(DavixError **err){
 }
 
 void NEONRequest::configureRequest(){
-    // reconfigure protos
-    configureRequestParamsProto(*_current, params);
 
-    std::copy(params.getHeaders().begin(), params.getHeaders().end(), std::back_inserter(_headers_field));
+    // add custom user headers, but make sure they're only added once
+    // in case of a redirect
+    if(!req_running) {
+        std::copy(params.getHeaders().begin(), params.getHeaders().end(), std::back_inserter(_headers_field));
+    }
 
-    // configure S3 params if needed
-    if(params.getProtocol() == RequestProtocol::AwsS3)
-        configureS3params();
+    /* Doing a PUT to Azure? Add the necessary Microsoft-specific header */
+    if(_request_type == "PUT" &&
+       _current.get()->queryParamExists("sig") &&
+       _current.get()->queryParamExists("sr") &&
+       _current.get()->queryParamExists("sp")) {
+
+        _headers_field.push_back(HeaderLine("x-ms-blob-type", "BlockBlob"));
+    }
 
     // setup timeout
     if(_expiration_time.isValid() == false
@@ -279,9 +305,14 @@ void NEONRequest::configureRequest(){
 
 
 void NEONRequest::configureS3params(){
-    HeaderVec vec = _headers_field;
-    S3::signRequest(params, _request_type, *_current, vec);
-    vec.swap(_headers_field);
+    Uri signed_url = S3::signURI(params, _request_type, *_current, _headers_field, NEON_S3_SIGN_DURATION);
+    _current= boost::shared_ptr<Uri>(new Uri(signed_url));
+}
+
+// TODO: make static?
+void NEONRequest::configureAzureParams(){
+    Uri signed_url = Azure::signURI(params.getAzureKey(), _request_type, *_current, NEON_S3_SIGN_DURATION);
+    _current= boost::shared_ptr<Uri>(new Uri(signed_url));
 }
 
 int NEONRequest::processRedirection(int neonCode, DavixError **err){
@@ -317,7 +348,7 @@ int NEONRequest::startRequest(DavixError **err){
 
 int NEONRequest::negotiateRequest(DavixError** err){
 
-    const int auth_retry_limit = 5;
+    const int auth_retry_limit = params.getOperationRetry();
     int code, status, end_status = NE_RETRY;
     _last_read = -1;
     _total_read_size = 0;
@@ -336,7 +367,7 @@ int NEONRequest::negotiateRequest(DavixError** err){
 
     req_started = req_running = true;
 
-    while(end_status == NE_RETRY && _number_try < auth_retry_limit){
+    while(end_status == NE_RETRY && _number_try <= auth_retry_limit) {
         DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, "NEON start internal request");
 
         if( (status = ne_begin_request(_req)) != NE_OK && status != NE_REDIRECT){
@@ -378,8 +409,17 @@ int NEONRequest::negotiateRequest(DavixError** err){
                 if( (end_status = processRedirection(status, err)) <0){
                    return -1;
                 }
+                _number_try--;
+                _redirects++;
+
+                if(_redirects > NEON_REDIRECT_LIMIT) {
+                    httpcodeToDavixError(code, davix_scope_http_request(), "Too many redirects", err);
+                    return -1;
+                }
+
                 break;
             case 401: // authentification requested, do retry
+            case 403:
                 ne_discard_response(_req);
                 end_status = ne_end_request(_req);
                 _last_read = -1;
@@ -396,9 +436,8 @@ int NEONRequest::negotiateRequest(DavixError** err){
                     }
 
                     if(end_status == NE_OK){
-                            DavixError::setupError(err,davix_scope_http_request(),
-                                                   StatusCode::AuthentificationError, "401 Unauthorized");
-                            return -1;
+                        httpcodeToDavixError(code, davix_scope_http_request(), "", err);
+                        return -1;
                     }
 
                     createError(end_status, err);
@@ -406,7 +445,6 @@ int NEONRequest::negotiateRequest(DavixError** err){
                 }
                 DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "(NEON) {} code -> request again ", code);
                 break;
-            case 403:
             case 501:
                  // cleanup redirection
                 _number_try++;
@@ -418,6 +456,9 @@ int NEONRequest::negotiateRequest(DavixError** err){
                 end_status = 0;
                 break;
             default:
+                if(code >= 400) {
+                    httpcodeToDavixError(code, davix_scope_http_request(), "", err);
+                }
                 end_status = 0;
                 break;
 
@@ -425,13 +466,12 @@ int NEONRequest::negotiateRequest(DavixError** err){
         _number_try++;
     }
 
-    if(_number_try >= auth_retry_limit){
-        if(_neon_sess.get() != NULL && _neon_sess->getLastError(err)){
-            DavixError::setupError(err,davix_scope_http_request(), StatusCode::AuthenticationError,
-                               "Authentication Error, reach maximum number of attempts");
-        }
+    if(end_status == NE_RETRY) {
+        DavixError::setupError(err,davix_scope_http_request(), StatusCode::AuthenticationError,
+                               "Authentication error, reached maximum number of attempts");
         return -2;
     }
+
     return 0;
 }
 
