@@ -93,6 +93,10 @@ struct RequestParamsInternal{
         _call_loginpswwd(NULL),
         _call_loginpswwd_userdata(NULL),
         _aws_cred(),
+        _aws_region(),
+        _aws_token(),
+        _aws_alternate(false),
+        _azure_key(),
         ops_timeout(),
         connexion_timeout(),
         agent_string(default_agent),
@@ -133,6 +137,10 @@ struct RequestParamsInternal{
         _call_loginpswwd(param_private._call_loginpswwd),
         _call_loginpswwd_userdata(param_private._call_loginpswwd_userdata),
         _aws_cred(param_private._aws_cred),
+        _aws_region(param_private._aws_region),
+        _aws_token(param_private._aws_token),
+        _aws_alternate(param_private._aws_alternate),
+        _azure_key(param_private._azure_key),
         ops_timeout(),
         connexion_timeout(),
         agent_string(param_private.agent_string),
@@ -170,6 +178,10 @@ struct RequestParamsInternal{
     authCallbackLoginPasswordBasic _call_loginpswwd;
     void* _call_loginpswwd_userdata;
     std::pair<AwsSecretKey, AwsAccessKey> _aws_cred;
+    AwsRegion _aws_region;
+    AwsToken _aws_token;
+    bool _aws_alternate;
+    AzureSecretKey _azure_key;
 
     // timeout management
     struct timespec ops_timeout;
@@ -330,6 +342,38 @@ const std::pair<AwsSecretKey, AwsAccessKey> & RequestParams::getAwsAutorizationK
     return d_ptr->_aws_cred;
 }
 
+void RequestParams::setAwsRegion(const AwsRegion &region) {
+    d_ptr->_aws_region = region;
+}
+
+const AwsRegion & RequestParams::getAwsRegion() const {
+    return d_ptr->_aws_region;
+}
+
+void RequestParams::setAwsToken(const AwsToken &token) {
+    d_ptr->_aws_token = token;
+}
+
+const AwsToken & RequestParams::getAwsToken() const {
+    return d_ptr->_aws_token;
+}
+
+void RequestParams::setAwsAlternate(const bool &alternate) {
+    d_ptr->_aws_alternate = alternate;
+}
+
+const bool & RequestParams::getAwsAlternate() const {
+    return d_ptr->_aws_alternate;
+}
+
+void RequestParams::setAzureKey(const AzureSecretKey &key) {
+    d_ptr->_azure_key = key;
+}
+
+const AzureSecretKey & RequestParams::getAzureKey() const {
+    return d_ptr->_azure_key;
+}
+
 void RequestParams::setS3ListingMode(const S3ListingMode::S3ListingMode s3_listing_mode){
     d_ptr->_s3_listing_mode = s3_listing_mode;
 }
