@@ -164,7 +164,7 @@ static int populateTaskQueue(Context& c, const Tool::OptParams & opts, std::stri
     if(tmp_err){
         Tool::errorPrint(&tmp_err);
         std::cerr << std::endl << "Error occured during listing  " << dirQueue.front() << " Number of entries processed in current directory: " << entry_counter << ". Continuing..."<< std::endl;
-        std::cerr << std::endl << "Last succesful entry is " << last_success_entry << std::endl;
+        std::cerr << std::endl << "Last successful entry is " << last_success_entry << std::endl;
     }
 
     pos.closedirpp(fd, NULL);
