@@ -54,6 +54,10 @@ const std::string scope_params = "Davix::Tools::Params";
 #define HAS_INPUT_FILE      1017
 #define THIRD_PT_COPY_MODE  1018
 #define DISABLE_LISTING_CAP 1019
+#define S3_REGION           1020
+#define S3_ALTERNATE        1021
+#define AZURE_KEY           1022
+#define S3_TOKEN            1023
 
 // LONG OPTS
 
@@ -81,6 +85,10 @@ const std::string scope_params = "Davix::Tools::Params";
 {"userpass", required_argument, 0, USER_PASSWORD}, \
 {"s3secretkey", required_argument, 0, S3_SECRET_KEY}, \
 {"s3accesskey", required_argument, 0, S3_ACCESS_KEY}, \
+{"s3region", required_argument, 0, S3_REGION}, \
+{"s3token", required_argument, 0, S3_TOKEN}, \
+{"azurekey", required_argument, 0, AZURE_KEY}, \
+{"s3alternate", no_argument, 0, S3_ALTERNATE}, \
 {"insecure", no_argument, 0,  'k' }
 
 #define REQUEST_LONG_OPTIONS \
@@ -112,6 +120,9 @@ OptParams::OptParams() :
     userlogpasswd(),
     req_content(),
     aws_auth(),
+    aws_region(),
+    aws_token(),
+    aws_alternate(false),
     pres_flag(0),
     shell_flag(0),
     has_input_file(false),
@@ -123,7 +134,7 @@ OptParams::OptParams() :
 
 static void option_abort(char** argv){
     std::cerr << argv[0] <<", Error: Wrong argument format\n";
-    std::cerr << "Try '" << argv[0] <<" --help' for more informations" << std::endl;
+    std::cerr << "Try '" << argv[0] <<" --help' for more information" << std::endl;
     exit(-1);
 }
 
@@ -234,12 +245,14 @@ int parse_davix_options_generic(const std::string &opt_filter,
                 break;
             case USER_LOGIN:
                 p.userlogpasswd.first = optarg;
+                strncpy(optarg, "", strlen(optarg));
                 break;
             case X509_PRIVATE_KEY:
                 p.priv_key = SanitiseTildedPath(optarg).c_str();
                 break;
             case USER_PASSWORD:
                 p.userlogpasswd.second = optarg;
+                strncpy(optarg, "", strlen(optarg));
                 break;
             case DATA_CONTENT:
                 p.req_content = optarg;
@@ -252,9 +265,25 @@ int parse_davix_options_generic(const std::string &opt_filter,
                 break;
             case S3_ACCESS_KEY:
                 p.aws_auth.second = optarg;
+                strncpy(optarg, "", strlen(optarg));
                 break;
             case S3_SECRET_KEY:
                 p.aws_auth.first = optarg;
+                strncpy(optarg, "", strlen(optarg));
+                break;
+            case S3_REGION:
+                p.aws_region = optarg;
+                break;
+            case S3_TOKEN:
+                p.aws_token = optarg;
+                strncpy(optarg, "", strlen(optarg));
+                break;
+            case S3_ALTERNATE:
+                p.aws_alternate = true;
+                break;
+            case AZURE_KEY:
+                p.azure_key = optarg;
+                strncpy(optarg, "", strlen(optarg));
                 break;
             case 'l':
                 p.pres_flag |= LONG_LISTING_FLAG;
@@ -482,6 +511,11 @@ std::string get_common_options(){
             "\t--userpass:               User password for login/password authentication\n"
             "\t--s3secretkey SEC_KEY:    S3 authentication: secret key\n"
             "\t--s3accesskey ACC_KEY:    S3 authentication: access key\n"
+            "\t--s3region REGION:        S3 region (optional - if passed, will authenticate using a v4 signature instead of v2)\n"
+            "\t--s3token TOKEN:          S3 security token - used along with STS temporary credentials\n"
+            "\t--s3alternate:            Pass this flag if you're using a path-based S3 URL\n"
+            "\t                          A path-based URL contains the bucket name in the path, ie https://s3-someregion.amazonaws.com/mybucket/file\n"
+            "\t--azurekey AZURE_KEY:     Azure authentication secret key\n"
             ;
 }
 
