@@ -66,6 +66,14 @@ struct OptParams{
     std::string req_content;
     // s3 auth
     std::pair<std::string, std::string> aws_auth;
+    // s3 region
+    std::string aws_region;
+    // s3 security token
+    std::string aws_token;
+    // s3 alternate url
+    bool aws_alternate;
+    // azure key
+    std::string azure_key;
     // presentation flag
     int pres_flag;
     // shell flags
