@@ -103,9 +103,18 @@ int configureAuth(OptParams & opts){
     //setup aws creds
     if(opts.aws_auth.first.empty() == false){
         opts.params.setAwsAuthorizationKeys(opts.aws_auth.first, opts.aws_auth.second);
+        opts.params.setAwsRegion(opts.aws_region);
+        opts.params.setAwsToken(opts.aws_token);
+        opts.params.setAwsAlternate(opts.aws_alternate);
         opts.params.setProtocol(RequestProtocol::AwsS3);
     }
 
+    // setup azure creds
+    if(opts.azure_key.empty() == false) {
+        opts.params.setAzureKey(opts.azure_key);
+        opts.params.setProtocol(RequestProtocol::Azure);
+    }
+
     return 0;
 }
 
@@ -232,8 +241,8 @@ int authCallbackLoginPassword(void* userdata, const SessionInfo & info, std::str
                 ret =0;
             }
         }
+        std::cout << std::endl;
     }
-    std::cout << std::endl;
     if(ret < 0)
         DavixError::setupError(err, "Davix::Tool::Auth",
                                StatusCode::LoginPasswordError,
@@ -278,8 +287,8 @@ int authCallbackCert(void* userdata, const SessionInfo & info, X509Credential* c
                 }
                 return -1;
              }
+             std::cout << std::endl;
         }
-        std::cout << std::endl;
         return 0;
     }
     if(err && *err == NULL){
